__all__ = [
    'fastqc_caller',
    'multiQC_report',
    'generate_DE',
    'RNAbiotype',
    'mapReads',
    'cutadapt_caller',
    'mirtop_caller',
    'optimir_caller',
    'miraligner_caller',
    'MINTMap_caller'
    
]

from XICRA.scripts import *
